"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const execa = require("execa");
const fs = require("fs");
const rimraf = require("rimraf");
const path = require("path");
const mkdirp = require("mkdirp");
const util_1 = require("./util");
function installDependencies(projectDir, project, opts) {
    const args = ['install', '--ignore-scripts'];
    if (opts.force) {
        args.push('--force');
    }
    execa.sync(project.npmClient, args, { cwd: projectDir });
    util_1.logger.iterateInfo(`installed dependencies for ${project.package}`, opts.countOf);
}
exports.installDependencies = installDependencies;
function buildProject(projectDir, project, opts) {
    if (project.lerna) {
        execa.sync('lerna', ['run', 'build'], { cwd: projectDir });
        util_1.logger.iterateInfo(`built ${project.package}`, opts.countOf);
        return;
    }
    execa.sync(project.npmClient, [project.buildScript], { cwd: projectDir });
    util_1.logger.iterateInfo(`built ${project.package}`, opts.countOf);
    return;
}
exports.buildProject = buildProject;
function linkSelf(projectDir, project, opts) {
    let cwd = projectDir;
    if (project.linkFrom) {
        cwd = path.join(projectDir, project.linkFrom);
    }
    try {
        if (!project.lerna) {
            execa.sync(project.npmClient, ['unlink'], { cwd });
            util_1.logger.iterateInfo(`unlinking ${project.package}`, opts.countOf);
        }
        else {
            execa.sync('lerna', ['exec', project.npmClient, 'unlink'], { cwd });
            util_1.logger.iterateInfo(`unlinking subpackages in ${project.package}`, opts.countOf);
        }
    }
    catch (err) {
        util_1.logger.warn(`No registered package found for "${project.package}" yet. Linking now...`);
    }
    if (!project.lerna) {
        execa.sync(project.npmClient, ['link'], { cwd });
        util_1.logger.iterateInfo(`linked ${project.package}`, opts.countOf);
    }
    else {
        execa.sync('lerna', ['exec', project.npmClient, 'link'], { cwd });
        util_1.logger.iterateInfo(`linked subpackges in ${project.package}`, opts.countOf);
    }
    return;
}
exports.linkSelf = linkSelf;
function linkDependencies(projectDir, project, opts) {
    if (project.links) {
        if (project.lerna) {
            execa.sync('lerna', ['exec', '--', project.npmClient, 'link', ...project.links], {
                cwd: projectDir,
            });
            util_1.logger.iterateInfo(`linked subpackage dependencies in ${project.package}`, opts.countOf);
        }
        else {
            execa.sync(project.npmClient, ['link', ...project.links], {
                cwd: projectDir,
            });
            util_1.logger.iterateInfo(`linked dependencies of ${project.package}`, opts.countOf);
        }
        return;
    }
}
exports.linkDependencies = linkDependencies;
function cloneTypeDefinitions(baseDir, typeDefs) {
    for (let [typeDefName, typeInfo] of Object.entries(typeDefs)) {
        cloneTypeDefinition(baseDir, typeDefName, typeInfo);
    }
}
exports.cloneTypeDefinitions = cloneTypeDefinitions;
function cloneTypeDefinition(baseDir, typeDefName, typeInfo) {
    const typeDefPath = path.join(baseDir, '@types', typeDefName);
    if (!fs.existsSync(typeDefName)) {
        mkdirp.sync(typeDefName);
    }
    if (util_1.repoExists(typeDefPath)) {
        util_1.logger.warn(`Type repository for ${typeDefName} is present`);
        checkoutBranch(typeDefPath, typeInfo.branch);
        return;
    }
    util_1.logger.warn('setting up typeDefs, this could take a while...');
    execa.sync('git', [
        'clone',
        typeInfo.remote,
        typeDefPath,
        '--branch',
        typeInfo.branch,
        '--depth',
        typeInfo.depth.toString(),
    ]);
    return util_1.logger.info(`cloned typeDefinition for ${typeDefName}`);
}
exports.cloneTypeDefinition = cloneTypeDefinition;
function linkType(type, projectDir, baseDir, typeDef) {
    const typeSymbolicPath = path.join(projectDir, `node_modules/@types/${type}`);
    const typePath = path.join(baseDir, '@types', typeDef.typesPath);
    rimraf.sync(typeSymbolicPath);
    fs.symlinkSync(typePath, typeSymbolicPath, 'dir');
}
exports.linkType = linkType;
function linkTypes(projectDir, project, config) {
    const { baseDir, typeDefs } = config;
    if (project.types) {
        for (let type of project.types) {
            linkType(type, projectDir, baseDir, typeDefs[type]);
        }
    }
}
exports.linkTypes = linkTypes;
function cloneProject(remote, projectDir) {
    if (!util_1.repoExists(projectDir)) {
        return execa.sync('git', ['clone', remote, projectDir]);
    }
    return;
}
exports.cloneProject = cloneProject;
function checkoutBranch(projectDir, branch) {
    if (!util_1.repoExists(projectDir)) {
        throw Error(`Cannot checkout ${branch}, ${projectDir} is not a git repository`);
    }
    return execa.sync('git', ['checkout', branch], { cwd: projectDir });
}
exports.checkoutBranch = checkoutBranch;
function cloneAndInstall(projectDir, project, opts) {
    cloneProject(project.remote, projectDir);
    checkoutBranch(projectDir, project.branch);
    util_1.logger.iterateInfo(`cloned and checked out ${project.package}#${project.branch}`, opts.countOf);
    installDependencies(projectDir, project, opts);
}
exports.cloneAndInstall = cloneAndInstall;
